@@ -10,6 +10,7 @@ except Exception, e:
     VERSION = 'unknown'
 
 from django import forms
+from django.db.models.sql.expressions import SQLEvaluator
 from django.db.models.fields import Field, BigIntegerField
 from django.db.models.fields.subclassing import Creator, LegacyConnection
 
@@ -19,41 +20,102 @@ class Bit(object):
     """
     def __init__(self, number, is_set=True):
         self.number = number
-        self.is_set = is_set
-    
+        self.is_set = bool(is_set)
+        self.mask = 2**int(number)
+        if not self.is_set:
+            self.mask = ~self.mask
+
     def __repr__(self):
-        return '<%s: %s>' % (self.__class__.__name__, self.number)
-    
-    def __str__(self):
-        if self.is_set:
-            return 'Yes'
-        return 'No'
-    
+        return '<%s: number=%d, is_set=%s>' % (self.__class__.__name__, self.number, self.is_set)
+
+    # def __str__(self):
+    #     if self.is_set:
+    #         return 'Yes'
+    #     return 'No'
+
     def __int__(self):
-        return int(self.is_set)
-    
+        return self.mask
+
     def __nonzero__(self):
         return self.is_set
-        
+
     def __eq__(self, value):
         if isinstance(value, Bit):
             return value.number == self.number and value.is_set == self.is_set
-        return bool(value) == self.is_set
+        elif isinstance(value, bool):
+            return value == self.is_set
+        elif isinstance(value, int):
+            return value == self.mask
+        return value == self.is_set
 
     def __ne__(self, value):
-        if isinstance(value, Bit):
-            return value != self
-        return bool(value) != self.is_set
+        return not self == value
 
     def __coerce__(self, value):
         return (self.is_set, bool(value))
 
     def __invert__(self):
-        return Bit(self.number, bool(not self.is_set))
+        return self.__class__(self.number, not self.is_set)
+
+    def __and__(self, value):
+        if isinstance(value, Bit):
+            value = value.mask
+        return value & self.mask
+
+    def __rand__(self, value):
+        if isinstance(value, Bit):
+            value = value.mask
+        return self.mask & value
+
+    def __or__(self, value):
+        if isinstance(value, Bit):
+            value = value.mask
+        return value | self.mask
+
+    def __ror__(self, value):
+        if isinstance(value, Bit):
+            value = value.mask
+        return self.mask | value
+
+    def __lshift__(self, value):
+        if isinstance(value, Bit):
+            value = value.mask
+        return value << self.mask
+
+    def __rlshift__(self, value):
+        if isinstance(value, Bit):
+            value = value.mask
+        return self.mask << value
+
+    def __rshift__(self, value):
+        if isinstance(value, Bit):
+            value = value.mask
+        return value >> self.mask
+
+    def __rrshift__(self, value):
+        if isinstance(value, Bit):
+            value = value.mask
+        return self.mask >> value
+
+    def __xor__(self, value):
+        if isinstance(value, Bit):
+            value = value.mask
+        return value ^ self.mask
+
+    def __rxor__(self, value):
+        if isinstance(value, Bit):
+            value = value.mask
+        return self.mask ^ value
 
     def __sentry__(self):
         return repr(self)
 
+    def prepare(self, evaluator, query, allow_joins):
+        return self
+
+    def evaluate(self, evaluator, qn, connection):
+        return self.mask, []
+
 class BitHandler(object):
     """
     Represents an array of bits, each as a ``Bit`` object.
@@ -114,7 +176,7 @@ class BitHandler(object):
         if key not in self._keys:
             raise AttributeError('%s is not a valid flag' % key)
         return self.get_bit(self._keys.index(key))
-    
+
     def __setattr__(self, key, value):
         if key.startswith('_'):
             return object.__setattr__(self, key, value)
@@ -122,13 +184,16 @@ class BitHandler(object):
             raise AttributeError('%s is not a valid flag' % key)
         self.set_bit(self._keys.index(key), value)
 
+    def __iter__(self):
+        return self.iteritems()
+
     def __sentry__(self):
         return repr(self)
 
     def get_bit(self, bit_number):
         mask = 2**int(bit_number)
         return Bit(bit_number, self._value & mask != 0)
-    
+
     def set_bit(self, bit_number, true_or_false):
         mask = 2**int(bit_number)
         if true_or_false:
@@ -136,9 +201,20 @@ class BitHandler(object):
         else:
             self._value &= (~mask)
         return Bit(bit_number, self._value & mask != 0)
-    
+
     def keys(self):
         return self._keys
+    
+    def iterkeys(self):
+        return iter(self._keys)
+        
+    def items(self):
+        return list(self.iteritems())
+
+    def iteritems(self):
+        for k in self._keys:
+            yield (k, getattr(self, k).is_set)
+    
 
 class BitFormField(forms.IntegerField):
     def __init__(self, *args, **kwargs):
@@ -151,31 +227,31 @@ class BitFormField(forms.IntegerField):
 class BitFieldFlags(object):
     def __init__(self, flags):
         self._flags = flags
-    
+
     def __repr__(self):
         return repr(self._flags)
-    
+
     def __iter__(self):
         for flag in self._flags:
             yield flag
-    
+
     def __getattr__(self, key):
         if key not in self._flags:
             raise AttributeError
         return Bit(self._flags.index(key))
-    
+
     def iteritems(self):
         for flag in self._flags:
-            yield flag, Bit(self._flags.index(key))
-    
+            yield flag, Bit(self._flags.index(flag))
+
     def iterkeys(self):
         for flag in self._flags:
             yield flag
-        
+
     def itervalues(self):
         for flag in self._flags:
-            yield Bit(self._flags.index(key))
-    
+            yield Bit(self._flags.index(flag))
+
     def items(self):
         return list(self.iteritems())
 
@@ -201,24 +277,6 @@ class BitFieldCreator(Creator):
             retval._keys = self.field.flags
         return retval
 
-class BitFieldMeta(LegacyConnection):
-    """
-    Modified SubFieldBase to use our contribute_to_class method (instead of
-    monkey-patching make_contrib).  This uses our BitFieldCreator descriptor
-    in place of the default.
-
-    NOTE: If we find ourselves needing custom descriptors for fields, we could
-    make this generic.
-    """
-    def __new__(cls, name, bases, attrs):
-        def contribute_to_class(self, cls, name):
-            super(self.__class__, self).contribute_to_class(cls, name)
-            setattr(cls, self.name, BitFieldCreator(self))
-
-        new_class = super(BitFieldMeta, cls).__new__(cls, name, bases, attrs)
-        new_class.contribute_to_class = contribute_to_class
-        return new_class
-
 class BitQueryLookupWrapper(object):
     def __init__(self, alias, column, bit):
         self.table_alias = alias
@@ -232,13 +290,11 @@ class BitQueryLookupWrapper(object):
 
         This will be called by Where.as_sql()
         """
-        mask = 2**self.bit.number
         if self.bit:
-            return ("(%s.%s | %d)" % (qn(self.table_alias), qn(self.column), mask),
+            return ("(%s.%s | %d)" % (qn(self.table_alias), qn(self.column), self.bit.mask),
                     [])
-        return ("(%s.%s & ~%d)" % (qn(self.table_alias), qn(self.column), mask),
+        return ("(%s.%s & %d)" % (qn(self.table_alias), qn(self.column), self.bit.mask),
                 [])
-        
 
 class BitQuerySaveWrapper(BitQueryLookupWrapper):
     def as_sql(self, qn, connection):
@@ -255,19 +311,36 @@ class BitQuerySaveWrapper(BitQueryLookupWrapper):
             raise NotImplementedError
         else:
             XOR_OPERATOR = '^'
-        
-        mask = 2**self.bit.number
+
         if self.bit:
-            return ("%s.%s | %d" % (qn(self.table_alias), qn(self.column), mask),
+            return ("%s.%s | %d" % (qn(self.table_alias), qn(self.column), self.bit.mask),
                     [])
-        return ("%s.%s %s %d" % (qn(self.table_alias), qn(self.column), XOR_OPERATOR, mask),
+        return ("%s.%s %s %d" % (qn(self.table_alias), qn(self.column), XOR_OPERATOR, self.bit.mask),
                 [])
 
+class BitFieldMeta(LegacyConnection):
+    """
+    Modified SubFieldBase to use our contribute_to_class method (instead of
+    monkey-patching make_contrib).  This uses our BitFieldCreator descriptor
+    in place of the default.
+
+    NOTE: If we find ourselves needing custom descriptors for fields, we could
+    make this generic.
+    """
+    def __new__(cls, name, bases, attrs):
+        def contribute_to_class(self, cls, name):
+            BigIntegerField.contribute_to_class(self, cls, name)
+            setattr(cls, self.name, BitFieldCreator(self))
+
+        new_class = super(BitFieldMeta, cls).__new__(cls, name, bases, attrs)
+        new_class.contribute_to_class = contribute_to_class
+        return new_class
+
 class BitField(BigIntegerField):
     __metaclass__ = BitFieldMeta
 
     def __init__(self, flags, *args, **kwargs):
-        super(BitField, self).__init__(*args, **kwargs)
+        BigIntegerField.__init__(self, *args, **kwargs)
         self.flags = flags
 
     def south_field_triple(self):
@@ -280,28 +353,40 @@ class BitField(BigIntegerField):
     def formfield(self, form_class=BitFormField, **kwargs):
         return Field.formfield(self, form_class, **kwargs)
 
+    def pre_save(self, instance, add):
+        value = getattr(instance, self.attname)
+        return value
+
     def get_prep_value(self, value):
+        if isinstance(value, Bit):
+            value = value.mask
         return int(value)
 
-    def get_db_prep_save(self, value, connection):
-        if isinstance(value, Bit):
-            return BitQuerySaveWrapper(self.model._meta.db_table, self.name, value)
-        return super(BitField, self).get_db_prep_save(value, connection=connection)
+    # def get_db_prep_save(self, value, connection):
+    #     if isinstance(value, Bit):
+    #         return BitQuerySaveWrapper(self.model._meta.db_table, self.name, value)
+    #     return super(BitField, self).get_db_prep_save(value, connection=connection)
 
     def get_db_prep_lookup(self, lookup_type, value, connection, prepared=False):
+        if isinstance(value, SQLEvaluator) and isinstance(value.expression, Bit):
+            value = value.expression
         if isinstance(value, Bit):
             return BitQueryLookupWrapper(self.model._meta.db_table, self.name, value)
-        return super(BitField, self).get_db_prep_lookup(lookup_type=lookup_type, value=value,
+        return BigIntegerField.get_db_prep_lookup(self, lookup_type=lookup_type, value=value,
                                                         connection=connection, prepared=prepared)
 
     def get_prep_lookup(self, lookup_type, value):
+        if isinstance(value, SQLEvaluator) and isinstance(value.expression, Bit):
+            value = value.expression
         if isinstance(value, Bit):
             if lookup_type in ('exact',):
                 return value
             raise TypeError('Lookup type %r not supported with `Bit` type.' % lookup_type)
-        return super(BitField, self).get_prep_lookup(lookup_type, value)
+        return BigIntegerField.get_prep_lookup(self, lookup_type, value)
 
     def to_python(self, value):
+        if isinstance(value, Bit):
+            value = value.mask
         if not isinstance(value, BitHandler):
             # Regression for #1425: fix bad data that was created resulting
             # in negative values for flags.  Compute the value that would
@@ -316,4 +401,4 @@ class BitField(BigIntegerField):
         else:
             # Ensure flags are consistent for unpickling
             value._keys = self.flags
-        return value
\ No newline at end of file
+        return value
